%  LIBERTY.m is the core of the LIBERTY model.
%  A Leaf Incorporating Biochemistry Exhibiting Reflectance and Transmittance Yields
%  LIBERTY.M was Compiled into Matlab code by xiguang Yang, NorthEast Forestry 
%  University, based on original LIBERTY Fortran code.The LIBETY.m was completed in May 2009.
% _______________________________________________________________________
% _______________________________________________________________________
%
%     Author: Yang Xiguang 
%     Organization: NorthEast Forestry University
%     Address: No.26 Hexing road,harbin,Heilongjiang Province, China
%     Postalcode: 150040
%     E-mail: yangxiguang21@163.com
%     Version: LIBERTY version 1.0 (Matlab) (25 May 2009)
%
% _______________________________________________________________________
%

function LRT=LIBERTY(D,xu,thick,baseline,element,c_factor,w_factor,l_factor,p_factor)
global data

% _______________________________________________________________________
%  * input variable explain as follow:
% ----------------------------------------------------------------------
%   Variable        Description                Typical values(and range) 
% ----------------------------------------------------------------------
%   Cell Diameter(D)   Average leaf cell diameter (m-6)      0 (20 - 100) 
% ----------------------------------------------------------------------
%   Intercellular       Determinant for the amount of                        
%   air space (xu)   radiative flux passing between cells 0.045 (0.01 - 0.1)
% ----------------------------------------------------------------------
%   Leaf thickness(thick)  Arbitrary value to determine single                 
%                           leaf reflectance and transmittance                              
%                          from infinite reflectance criteria 1.6 (1 - 10)  
% ----------------------------------------------------------------------
%   Baseline(baseline) Wavelength independent absorption to                
%    absorption        compensate for changes in absolute   Fresh: 0.0006  
%   	                     reflectance                    Dry: 0.0004  
% ----------------------------------------------------------------------
%   Albino          Absorption in the visible region due                  
%   absorption(element)      to lignin                      2 (0 - 4)    
% ----------------------------------------------------------------------
%   Chlorophyll              Chlorophyll (pigment)                                          
%   content (c_factor)         content (mg.m-2)             200 (0 - 600) 
% ----------------------------------------------------------------------
%    Water                                                                
%   content(w_factor)         Water content (g.m-2)         100 (0 - 500)
% ----------------------------------------------------------------------
%   Lignin and                                                            
%   Cellulose            Combined lignin and cellulose                         
%   content (l_factor)        content (g.m-2)               40 (10 - 80)  
% ----------------------------------------------------------------------
%   Nitrogen              Nitrogen content                                      
%   content (p_factor)        (g.m-2)                       1 (0.3 - 2.0) 
% ----------------------------------------------------------------------

spectraldata
% ----------------------------------------------------------------------
%	/* reading in pigment absorption coefficients... */
 k_chloro=data(:,1);
% ----------------------------------------------------------------------
%  /* Now to read in the absorption data... */
 k_water=data(:,2);
 
% ----------------------------------------------------------------------
%  /* now to read in trace element absorption coefficients... */
  ke=data(:,3);
% ----------------------------------------------------------------------
%  /* now to read in cellulose and lignin absorption coefficients... */
 k_ligcell=data(:,4);
% ----------------------------------------------------------------------
% /* now to read in protein absorption coefficients... */
  k_protein=data(:,5);;
% ----------------------------------------------------------------------
%  /* set the following loop for 420 (to 2500nm) or 120 (1000nm) */
for i=1:421
%     /* determining the total absorption coefficient... */
      coeff=D*(baseline+(k_chloro(i)*c_factor)+(k_water(i)*w_factor)+(ke(i)*element)+(k_ligcell(i)*l_factor)+(k_protein(i)*p_factor));

% 	  /* change of refractive index over wavelength... */
	  N1=1.4891-(0.0005*(i-1));
      N0=1.0;
      in_angle=59;
      % /* Index of refraction */
      alpha=in_angle*pi/180;
      beta=asin((N0/N1)*sin(alpha));
      para_r=(tan(alpha-beta))/(tan(alpha+beta));
      vert_r=-(sin(alpha-beta))/(sin(alpha+beta));
      
      me=0;
      width=pi/180;
      for p=1:90
          alpha=p*pi/180;
          beta=asin(N0/N1*sin(alpha));
          plus=alpha+beta;
          dif=alpha-beta;
          refl=0.5*((sin(dif))^2/(sin(plus))^2+(tan(dif))^2/(tan(plus))^2);
          me=me+(refl*sin(alpha)*cos(alpha)*width);
      end
      me=me*2;
      
     mi=0;
     mint=0;
     width=pi/180;
     critical=asin(N0/N1)*180/pi;
     for j=1:critical
         alpha=j*pi/180;
         beta=asin(N0/N1*sin(alpha));
         plus=alpha+beta;
         dif=alpha-beta;
         refl=0.5*((sin(dif))^2/(sin(plus))^2+(tan(dif))^2/(tan(plus))^2);
         mint=mint+refl*sin(alpha)*cos(alpha)*width;
     end
     mi=(1-sin(critical*pi/180)^2)+2*mint;
     
      M=2*(1-(coeff+1)*exp(-coeff))/coeff^2;
	  T=((1-mi)*M)/(1-(mi*M));
      x=xu/(1-(1-2*xu)*T);
	  a=me*T+x*T-me-T-x*me*T;
      b=1+x*me*T-2*x^2*me^2*T;
      c=2*me*x^2*T-x*T-2*x*me;
      R=0.5;% /* initial guess */
      for iterations=1:50
          next_R=-(a*R^2+c)/b;
          R=next_R;
      end
      
% 	  /* The next bit works out transmittance based upon Benford...*/
% 	  /* setting up unchanging parameters... */
	  rb=(2*x*me)+(x*T)-(x*T*2*x*me);
	  tb=sqrt(((R-rb)*(1-(R*rb)))/R);
	  whole=fix(thick);
      fraction=thick-whole;%
% 	/* The next bit works out the fractional value */
% 	/* for the interval between 1 and 2... */
    top=tb^(1+fraction)*((((1+tb)^2)-rb^2)^(1-fraction));
    bot1=(1+tb)^(2*(1-fraction))-rb^2;
    bot2=1+(64/3)*fraction*(fraction-0.5)*(fraction-1)*0.001;
    tif=top/(bot1*bot2);
    rif=(1+rb^2-tb^2-sqrt((1+rb^2-tb^2)^2-4*rb^2*(1-tif^2)))/(2*rb);    
%   /* Now to work out for integral thickness greater than 2 ... */
	if (whole>=2) 
        prev_t=1;
		prev_r=0;
		for step=1:(whole-1)
           cur_t=(prev_t*tb)/(1-(prev_r*rb));
		   cur_r=prev_r+(((prev_t*prev_t)*rb)/(1-(prev_r*rb)));
		   prev_t=cur_t;
		   prev_r=cur_r;
        end
    else
        cur_t=1;
		cur_r=0;
    end
    trans=(cur_t*tif)/(1-(rif*cur_r));
	refl=cur_r+((cur_t^2*rif)/(1-(rif*cur_r)));
    
    
      transR(i)=trans;
      reflR(i)=refl;
      wavelength(i)=400+((i-1)*5);
      RR(i)=R; 
    end
LRT(:,1)=wavelength;
LRT(:,2)=RR;
LRT(:,3)=reflR;
LRT(:,4)=transR;
end
% ======================================================================
 
